// 002 u_c_search_url_list
// 24 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_search_url_list;
  interface
    uses Classes, u_c_basic_object;

    type c_search_url= // one "search_url"
                      Class(c_basic_object)
                        // -- m_name: the IP
                        m_domain_name: String;
                        m_whois_server_key: String;
                        m_whois_answer_file_name: String;

                        m_country: String;
                        m_org_name: String; 

                        Constructor create_search_url(p_name, p_domain_name: String);
                        function f_display_search_url: String;
                        function f_c_self: c_search_url;
                        Destructor Destroy; Override;
                      end; // c_search_url

         c_search_url_list= // "search_url" list
                           Class(c_basic_object)
                             m_c_search_url_list: tStringList;

                             Constructor create_search_url_list(p_name: String);

                             function f_search_url_count: Integer;
                             function f_c_search_url(p_search_url_index: Integer): c_search_url;
                             function f_index_of(p_search_url_name: String): Integer;
                             function f_c_find_by_search_url(p_search_url_name: String): c_search_url;
                             procedure add_search_url(p_search_url_name: String; p_c_search_url: c_search_url);
                             function f_c_add_search_url(p_search_url_name, p_domain_name: String): c_search_url;
                             function f_c_add_unique_search_url(p_search_url_name, p_domain_name: String): c_search_url;
                             procedure display_search_url_list;
                             procedure save_to_file(p_full_file_name: String);

                             Destructor Destroy; Override;
                           end; // c_search_url_list

  implementation
    uses SysUtils, u_c_display;

    // -- c_search_url

    Constructor c_search_url.create_search_url(p_name, p_domain_name: String);
      begin
        Inherited create_basic_object(p_name);
        m_domain_name:= p_domain_name;
      end; // create_search_url

    function c_search_url.f_display_search_url: String;
      begin
        Result:= Format('%-15s ', [m_name])+ m_domain_name;
      end; // f_display_search_url

    function c_search_url.f_c_self: c_search_url;
      begin
        Result:= Self;
      end; // f_c_self

    Destructor c_search_url.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_search_url_list

    Constructor c_search_url_list.create_search_url_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_search_url_list:= tStringList.Create;
      end; // create_search_url_line

    function c_search_url_list.f_search_url_count: Integer;
      begin
        Result:= m_c_search_url_list.Count;
      end; // f_search_url_count

    function c_search_url_list.f_c_search_url(p_search_url_index: Integer): c_search_url;
      begin
        Result:= c_search_url(m_c_search_url_list.Objects[p_search_url_index]);
      end; //  f_c_search_url

    function c_search_url_list.f_index_of(p_search_url_name: String): Integer;
      begin
        Result:= m_c_search_url_list.IndexOf(p_search_url_name);
      end; // f_index_of

    function c_search_url_list.f_c_find_by_search_url(p_search_url_name: String): c_search_url;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_search_url_name);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_search_url(m_c_search_url_list.Objects[l_index_of]);
      end; // f_c_find_by_name

    procedure c_search_url_list.add_search_url(p_search_url_name: String; p_c_search_url: c_search_url);
      begin
        m_c_search_url_list.AddObject(p_search_url_name, p_c_search_url);
      end; // add_search_url

    function c_search_url_list.f_c_add_search_url(p_search_url_name, p_domain_name: String): c_search_url;
      begin
        Result:= c_search_url.create_search_url(p_search_url_name, p_domain_name);
        add_search_url(p_search_url_name, Result);
      end; // f_c_add_search_url

    function c_search_url_list.f_c_add_unique_search_url(p_search_url_name, p_domain_name: String): c_search_url;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_search_url_name);
        if l_index_of>= 0
          then Result:= f_c_search_url(l_index_of)
          else Result:= f_c_add_search_url(p_search_url_name, p_domain_name);
      end; // f_c_add_unique_search_url

    procedure c_search_url_list.display_search_url_list;
      var l_search_url_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_search_url_count));

        for l_search_url_index:= 0 to f_search_url_count- 1 do
          with f_c_search_url(l_search_url_index) do
            // display(f_c_search_url(l_search_url_index).f_display_search_url);
            display(Format('%-15s ', [m_name])+ ' '+ m_whois_answer_file_name);
      end; // display_search_url_list

    procedure c_search_url_list.save_to_file(p_full_file_name: String);
      var l_search_url_index: Integer;
      begin
        with tStringList.Create do
        begin
          for l_search_url_index:= 0 to f_search_url_count- 1 do
            with f_c_search_url(l_search_url_index) do
              // Add(Format('%-15s ', [m_name])+ ' '+ m_domain_name+ ' '+ m_whois_answer_file_name);
              Add(Format('%-15s ', [m_name])+ ' '+ m_whois_answer_file_name);

          SaveToFile(p_full_file_name);
          Free;
        end;
     end; // save_to_file

   Destructor c_search_url_list.Destroy;
      var l_search_url_index: Integer;
      begin
        for l_search_url_index:= 0 to f_search_url_count- 1 do
          f_c_search_url(l_search_url_index).Free;
        m_c_search_url_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_search_url_list
    end. // u_c_search_url_list
