// 002 u_c_whois_client_socket
// 23 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_whois_client_socket;
  interface
    uses u_c_basic_object
        , u_c_log
        , u_c_base_socket_4
        , u_c_client_socket_4
        , u_c_byte_buffer
        ;

    const k_ip_port_whois= 43;
    type t_whois_socket_state= (e_unknonw_state,
                e_sent_lookup_state,
                e_after_lookup_state,

                e_sent_connect_state,
                e_after_connect_state,

                e_sent_whois_state,
                e_after_whois_state,

                e_sent_quit_state,
                e_after_quit_state,

                e_last_state
                );

         c_whois_client_socket= Class; // forward
         t_po_data_socket_event= procedure(p_c_whois_socket: c_whois_client_socket) of object;

         c_whois_client_socket= class(c_client_socket)
                                  m_trace_whois: Boolean;

                                  m_whois_socket_state: t_whois_socket_state;

                                  m_whois_server_ip: String;
                                  m_visitor_ip: String;

                                  m_on_after_whois_socket_error: t_po_data_socket_event;

                                  // m_on_after_whois_lookup: t_po_data_socket_event;
                                  m_on_after_whois_connect: t_po_data_socket_event;
                                  m_on_after_whois_data_reception: t_po_data_socket_event;
                                  m_on_after_whois_data_closed_by_remote: t_po_data_socket_event;

                                  Constructor create_whois_client_socket(p_name: String);

                                  // -- helpers
                                  function f_c_self: c_whois_client_socket;
                                  procedure set_problem(p_text: String; p_next: Integer);

                                  // -- debug
                                  procedure trace_whois(p_text: String);

                                  // -- socket events
                                  procedure _handle_after_error(p_c_base_socket: c_base_socket);

(*
                                  // -- visitor actions
                                  procedure do_lookup(p_whois_server_name: String);
                                  procedure _handle_after_lookup(p_c_client_socket: c_client_socket);
*)
                                  procedure do_connect(p_whois_server_ip: String; p_port: Integer);
                                  procedure _handle_after_connected(p_c_client_socket: c_client_socket);

                                  procedure do_search_domain(p_visitor_ip: String);
                                  procedure _handle_after_sent_whois(p_c_client_socket: c_client_socket);
                                  procedure _handle_after_received_data(p_c_client_socket: c_client_socket);
                                  procedure _handle_after_server_closed(p_c_client_socket: c_client_socket);

                                  Destructor Destroy; Override;
                                end; // c_whois_client_socket

  implementation
    uses SysUtils, TypInfo
        , u_c_display, u_types_constants, u_characters
       ;

    function f_whois_socket_state_name(p_whois_socket_state: t_whois_socket_state): String;
      begin
        Result:= GetEnumName(TypeInfo(t_whois_socket_state), Integer(p_whois_socket_state));
        Delete(Result, 1, 2);
        Delete(Result, Length(Result)- Length('_state')+ 1, Length('_state'));
      end; // f_whois_socket_state_name

    // -- c_whois_client_socket

    Constructor c_whois_client_socket.create_whois_client_socket(p_name: String);
      begin
        Inherited create_client_socket(p_name);

        m_trace_socket:= True;

        // -- link the events
        // -- all but reception are from c_client_socket
        m_on_after_socket_error:= Self._handle_after_error;

        // -- the socket winsock events
        // m_on_after_socket_host_lookup:= Self._handle_after_lookup;
        m_on_after_socket_connected:= Self._handle_after_connected;
        m_on_after_socket_received_data:= Self._handle_after_received_data;

        m_on_after_remote_server_client_socket_closed:= Self._handle_after_server_closed;

        wsa_startup;
      end; // create_whois

    function c_whois_client_socket.f_c_self: c_whois_client_socket;
      begin
        Result:= Self;
      end; // f_c_self

    procedure c_whois_client_socket.trace_whois(p_text: String);
      begin
        if m_trace_whois
          then display(p_text);
      end; // trace_whois

    procedure c_whois_client_socket.set_problem(p_text: String; p_next: Integer);
      begin
        display(p_text);
      end; // set_problem

    procedure c_whois_client_socket._handle_after_error(p_c_base_socket: c_base_socket);
        // -- the socket triggered an exception
        // --  - call on_after_reception to notify the user
        // --  - shutdown
      begin
        if p_c_base_socket= Nil
          then display_bug_halt('err_ p_c_client_socket Nil');

        with p_c_base_socket do
        begin
          display(m_socket_error_text);
          display(m_wsa_socket_error_message);

          if Assigned(m_on_after_whois_socket_error)
            then m_on_after_whois_socket_error(Self);
        end; // with p_c_base_socket
      end; // _handle_after_error

(*
    procedure c_whois_client_socket.do_lookup(p_whois_server_name: String);
      begin
        trace_whois('> whois_client.do_lookup '+ p_whois_server_name);
        m_whois_socket_state:= e_sent_lookup_state;
        do_lookup_win_socket_server(p_whois_server_name);
        trace_whois('< whois_client.do_lookup');
      end; // do_lookup

    procedure c_whois_client_socket._handle_after_lookup(p_c_client_socket: c_client_socket);
      begin
        m_whois_socket_state:= e_after_lookup_state;
        m_whois_server_ip:= m_remote_ip;
        if Assigned(m_on_after_whois_lookup)
          then m_on_after_whois_lookup(Self);
      end; // _handle_after_lookup
*)
    procedure c_whois_client_socket.do_connect(p_whois_server_ip: String; p_port: Integer);
      begin
        trace_whois('> whois_client.do_connect (to '+ p_whois_server_ip+ ', Port '+ IntToStr(p_port)+ ')');

        if m_whois_socket_state= e_after_lookup_state
          then begin
              display('create_win_socket');

              create_win_socket;
              wsa_select;
            end;

        m_whois_socket_state:= e_sent_connect_state;

        try
          m_c_reception_buffer.clear_buffer;
          do_connect_win_socket(p_whois_server_ip, p_port);
          // -- when connected, handle_connected will send the page request
        except
          display('*** c_whois_client_socket.do_connect Exception');
          _handle_after_error(Self);
        end;

        trace_whois('< whois_client.do_connect');
      end; // do_connect

    procedure c_whois_client_socket._handle_after_connected(p_c_client_socket: c_client_socket);
      begin
        trace_whois('> whois_client.after_connected');
        if Assigned(m_on_after_whois_connect)
          then m_on_after_whois_connect(Self);

        // -- now send the text
        m_c_reception_buffer.clear_buffer;
        m_whois_socket_state:= e_sent_whois_state;

        write_to_log('');
        write_to_log('W '+ m_visitor_ip);
        write_to_log('');
        f_do_send_string(m_visitor_ip+ k_new_line);

        trace_whois('< whois_client.after_connected');
      end; // _handle_after_connected

    procedure c_whois_client_socket._handle_after_received_data(p_c_client_socket: c_client_socket);
        // -- fd_read notification was received
        // --  - analyze the different answers to connect, etc
      var l_save_trace: Boolean;
      begin
        trace_whois('> whois_client._received_data '+ f_whois_socket_state_name(m_whois_socket_state));
        l_save_trace:= m_trace_whois;

        // -- only called when received some whois
        case m_whois_socket_state of
          e_sent_connect_state : _handle_after_connected(p_c_client_socket);
          e_sent_whois_state,
            e_after_whois_state : _handle_after_sent_whois(p_c_client_socket);
          e_sent_quit_state : ; // __handle_after_quit;

          else display_bug_stop('rec_whois_unknown state '+ IntToStr(Ord(m_whois_socket_state))
            + ' '+ f_whois_socket_state_name(m_whois_socket_state)+ '<');
        end; // case

        if l_save_trace
          then display('< whois_client._received_data ');
      end; // _handle_after_received_data

    procedure c_whois_client_socket.do_search_domain(p_visitor_ip: String);
      begin
        trace_whois('> do_search_domain '+ m_whois_server_ip);
        // -- mail box
        m_visitor_ip:= p_visitor_ip;

        create_win_socket;
        allocate_window;
        wsa_select;

        do_connect(m_whois_server_ip, k_ip_port_whois);
        trace_whois('< do_search_domain');
      end; // do_search_domain

    procedure c_whois_client_socket._handle_after_sent_whois(p_c_client_socket: c_client_socket);
      var l_save_trace: Boolean;
      begin
        trace_whois('> c_whois_client.after_sent_whois');
        l_save_trace:= m_trace_whois;
        m_whois_socket_state:= e_after_whois_state;
        if Assigned(m_on_after_whois_data_reception)
          then m_on_after_whois_data_reception(Self);
        if l_save_trace
          then display('< c_whois_client.after_sent_whois');
      end; // _handle_after_sent_whois

    procedure c_whois_client_socket._handle_after_server_closed(p_c_client_socket: c_client_socket);
      var l_save_trace: Boolean;
      begin
        trace_whois('> c_whois_client.after_server_closed');
        l_save_trace:= m_trace_whois;
        if Assigned(m_on_after_whois_data_closed_by_remote)
          then m_on_after_whois_data_closed_by_remote(Self);
        if l_save_trace
          then display('< c_whois_client.after_server_closed');
      end; // _handle_after_server_closed

    Destructor c_whois_client_socket.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_whois
    end. // u_c_whois
