// 002 u_c_whois_searcher
// 24 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_whois_searcher;
  interface
    uses Classes, u_c_basic_object
        , u_c_log
        , u_c_whois_client_socket
        , u_c_whois_server_list_3
      ;

    type c_whois_searcher= class; // forward
         t_on_whois_search_event= procedure(p_c_whois_searcher: c_whois_searcher) of Object;

         c_whois_searcher=
             class(c_basic_object)
               m_c_whois_server_list_ref: c_whois_server_list;
               m_c_traffic_log_ref: c_log;
               m_c_request_log_ref: c_log;

               m_search_ip: String;
               m_trial_count: Integer;

               m_c_result_string_list: tStringList;
               m_result_path, m_result_file_name: String;

               m_on_start_whois_search: t_on_whois_search_event;
               m_on_after_whois_search_connect: t_on_whois_search_event;
               m_on_after_whois_search_reception: t_on_whois_search_event;
               m_on_after_whois_search_server_closed: t_on_whois_search_event;

               Constructor create_whois_searcher(p_name: String;
                   p_c_traffic_log_ref: c_log;
                   p_c_whois_server_list_ref: c_whois_server_list);

               function f_whois_server_key: String;

               procedure search(p_search_ip: String);
               procedure handle_whois_searcher_socket_error(p_c_whois_socket: c_whois_client_socket);
               procedure handle_after_whois_searcher_connect(p_c_whois_socket: c_whois_client_socket);
               procedure handle_after_whois_searcher_reception(p_c_whois_socket: c_whois_client_socket);
               procedure handle_after_whois_searcher_server_closed(p_c_whois_socket: c_whois_client_socket);

               Destructor Destroy; Override;
             end; // c_whois_searcher

  implementation
    uses SysUtils, u_types_constants, u_characters, u_c_display
        , u_strings, u_exe
        ;

    // -- c_whois_searcher

    Constructor c_whois_searcher.create_whois_searcher(p_name: String;
        p_c_traffic_log_ref: c_log;
        p_c_whois_server_list_ref: c_whois_server_list);
      begin
        Inherited create_basic_object(p_name);

        m_c_traffic_log_ref:= p_c_traffic_log_ref;
        m_c_whois_server_list_ref:= p_c_whois_server_list_ref;

        m_c_whois_server_list_ref.reset_whois_servers;
        m_c_result_string_list:= tStringList.Create;
      end; // create_whois_searcher

    function c_whois_searcher.f_whois_server_key: String;
      begin
        Result:= m_c_whois_server_list_ref.m_c_current_whois_server.m_key;
      end; // f_whois_server_key

    procedure c_whois_searcher.search(p_search_ip: String);
        // -- send the domain whois request
        // -- use a local socket, to allow recursive calls
      var l_c_whois_search_client_socket: c_whois_client_socket;

      procedure create_search_socket;
          // -- create the socket
        begin
          l_c_whois_search_client_socket:= c_whois_client_socket.create_whois_client_socket('whois');

          with l_c_whois_search_client_socket do
          begin
            m_trace_whois:= True;
            m_c_traffic_log_ref:= m_c_traffic_log_ref;
            m_do_free_after_server_closed:= True;

            m_on_after_whois_socket_error:= handle_whois_searcher_socket_error;
            m_on_after_whois_connect:= handle_after_whois_searcher_connect;
            m_on_after_whois_data_reception:= handle_after_whois_searcher_reception;
            m_on_after_whois_data_closed_by_remote:= handle_after_whois_searcher_server_closed;
          end; // with m_c_whois_search_client_socket
        end; // create_search_socket

      function f_remove_blanks(p_ip: String): String;
        var l_index: Integer;
        begin
          Result:= '';
          for l_index:= 1 to Length(p_ip) do
            if p_ip[l_index]<> ' '
              then Result:= Result+ p_ip[l_index];
        end; // f_remove_blanks

      begin // search
        m_c_result_string_list.Clear;

        with m_c_whois_server_list_ref.m_c_current_whois_server do
          display('> search on '+ m_key+ ' '+ p_search_ip);

        if m_c_whois_server_list_ref.m_c_current_whois_server.m_unavailable
          then begin
              display('search on_timed_out. ');

              if not m_c_whois_server_list_ref.f_get_next_server
                then begin
                    display('*** no_other_available_server');
                    if Assigned(m_on_after_whois_search_server_closed)
                      then m_on_after_whois_search_server_closed(Self);
                    display('< search. Exit ');
                    Exit;
                  end
            end;

        m_search_ip:= f_remove_blanks(p_search_ip);

        if Assigned(m_on_start_whois_search)
          then m_on_start_whois_search(Self);

        m_c_request_log_ref.write_line(Format('%-30s', [m_search_ip])
            + f_whois_server_key);

        // -- create a brand new socket each time, since automatically closed
        // -- after search
        create_search_socket;
        Inc(m_trial_count);

        with l_c_whois_search_client_socket do
        begin
          m_whois_server_ip:= m_c_whois_server_list_ref.m_c_current_whois_server.m_ip;
          do_search_domain(m_search_ip);
        end;

        display('< search ');
      end; // search

    procedure c_whois_searcher.handle_whois_searcher_socket_error(p_c_whois_socket: c_whois_client_socket);
        // -- assume that only timeout errors. Anyway, remove the server from the list
      begin
        display('> handle_whois_searcher_socket_error');

        with p_c_whois_socket do
        begin
          m_c_whois_server_list_ref.current_whois_server_timed_out;
          // -- do not keep this server
          m_c_whois_server_list_ref.f_get_next_server;

          display(m_wsa_socket_error_message);
          display('*** time_out => get next ip');
          if Assigned(m_on_after_whois_search_server_closed)
            then m_on_after_whois_search_server_closed(Self);
        end;

        display('< handle_whois_searcher_socket_error');
      end; // handle_whois_searcher_socket_error

    procedure c_whois_searcher.handle_after_whois_searcher_connect(p_c_whois_socket: c_whois_client_socket);
      begin
        display('c_whois_server.handle_after_whois_connect');
        if Assigned(m_on_after_whois_search_connect)
          then m_on_after_whois_search_connect(Self);
      end; // handle_after_whois_searcher_connect

    procedure c_whois_searcher.handle_after_whois_searcher_reception(p_c_whois_socket: c_whois_client_socket);
        // -- could handle all in after_server_closed
      var l_index: Integer;
          l_the_line: String;
      begin
        display('> handle_after_whois_searcher_reception ');
        // -- load and anlyze the content

        with p_c_whois_socket, m_c_reception_buffer do
        begin
          l_the_line:= '';
          l_index:= m_read_index;
          while l_index< m_write_index do
          begin
             case m_oa_byte_buffer[l_index] of
               Ord(k_return) :
                   begin
                     m_c_result_string_list.Add(l_the_line);
                     l_the_line:= '';
                     If (l_index< m_write_index)
                         and (m_oa_byte_buffer[l_index]= Ord(k_line_feed))
                       then Inc(l_index);
                   end;
               Ord(k_line_feed) :
                   begin
                     m_c_result_string_list.Add(l_the_line);
                     l_the_line:= '';
                   end;
               else
                 l_the_line:= l_the_line+ Chr(m_oa_byte_buffer[l_index]);
             end; // case

             Inc(l_index)
          end; // while l_index

          m_read_index:= l_index;
        end; // with m_c_whois_search_client_socket

        m_c_result_string_list.Add(l_the_line);

        // -- Form update: led colors 
        if Assigned(m_on_after_whois_search_reception)
          then m_on_after_whois_search_reception(Self);

        display('< handle_after_whois_searcher_reception');
      end; // handle_after_whois_searcher_reception

    procedure c_whois_searcher.handle_after_whois_searcher_server_closed(p_c_whois_socket: c_whois_client_socket);
      var l_base_name: String;

      procedure save_result(p_segment, p_suffix: String);
        begin
          m_result_file_name:= p_segment+ l_base_name+ p_suffix;
          m_c_result_string_list.SaveToFile(m_result_path+ m_result_file_name);
        end; // save_result

      procedure handle_no_result;
        begin
          display('> no_result_test');
          m_c_request_log_ref.write_line('  '+ f_whois_server_key+ '<');
          m_result_file_name:= 'no_result';

          if Assigned(m_on_after_whois_search_server_closed)
            then m_on_after_whois_search_server_closed(Self);
          display('< no_result_test');
        end; // handle_no_result

      procedure test_server_switch;

        procedure search_another(p_save_name, p_log, p_new_key: String);
          begin
            save_result('wrong\', p_save_name);
            m_c_request_log_ref.write_line('  '+ p_log+ ' ');

            m_c_whois_server_list_ref.set_current_whois_server(p_new_key);
            // -- check that the recognized server has not timed out
            if m_c_whois_server_list_ref.m_c_current_whois_server.m_unavailable
              then begin
                  display('=== requested_timed_out. Get next ip for '+ m_search_ip);
                  m_c_request_log_ref.write_line('  timed '+ f_whois_server_key+ '<');

                  // -- start with another server
                  if not m_c_whois_server_list_ref.f_get_next_server
                    then begin
                        display('*** *** no_other_available_server');
                       end;

                  // -- not this is a recursive call
                  if Assigned(m_on_after_whois_search_server_closed)
                    then m_on_after_whois_search_server_closed(Self);
                end
              else begin
                  search(m_search_ip);
                end;
          end; // search_another

        procedure save_the_result;

          procedure do_save(p_check_signature: String);
            begin
              if Pos(p_check_signature, m_c_result_string_list.Text)> 0
                then save_result('', '.txt')
                else begin
                    save_result('', 'err.txt');
                    save_result('error\', '.txt');
                  end;
            end; // do_save

          var l_key: String;

          begin // save_the_result
            l_key:= f_whois_server_key;

            // -- check that the text contains the correct marker
            if l_key= 'us'
              then do_save('# ARIN WHOIS database') else
            if l_key= 'asia'
              then do_save('% [whois.apnic.net') else
            if l_key='latin'
              then do_save('% Copyright LACNIC lacnic.net') else
            if l_key= 'europe'
              then do_save('% This is the RIPE Whois query server') else
            if l_key= 'africa'
              then do_save('% This is the AfriNIC Whois server');
          end; // save_the_result

        begin // test_server_switch
(*
          if (f_whois_server_key= 'us') and (Pos('(NET-', m_c_result_string_list.Text)> 0)
            then begin
                l_position:= Pos('(NET-', m_c_result_string_list.Text);
              end else
*)
          if (f_whois_server_key= 'us') and (Pos('Allocated to RIPE', m_c_result_string_list.Text)> 0)
            then begin
                search_another('_to_europe.txt', 'us -> europe', 'europe');
              end else
          if (Pos('see http://www.iana.org', m_c_result_string_list.Text)> 0) // (f_whois_server_key= 'europe') and
              or  (Pos('Not assigned to LACNIC', m_c_result_string_list.Text)> 0)
            then begin
                search_another('_to_us.txt', 'europe -> us', 'us');
              end else
          if (f_whois_server_key= 'us') and (Pos('LACNIC-ARIN', m_c_result_string_list.Text)> 0)
            then begin
                search_another('_to_latin.txt', 'us -> latin', 'latin');
              end else
          if (f_whois_server_key= 'us') and (Pos('Allocated to APNIC', m_c_result_string_list.Text)> 0)
            then begin
                search_another('_to_asia.txt', 'us -> asia', 'asia');
              end else
          if (Pos('see http://www.afrinic.net', m_c_result_string_list.Text)> 0)
            then begin
                search_another('_to_africa.txt', ' -> africa', 'africa');
              end else
            begin
              // -- assumes no change needed. Save as is and get next
              // save_the_result;
              save_result('', '.txt');

              m_c_request_log_ref.write_line('  '+ f_whois_server_key+ '<');

              if Assigned(m_on_after_whois_search_server_closed)
                then m_on_after_whois_search_server_closed(Self);
            end;
        end; // test_server_switch

      begin // handle_after_whois_searcher_server_closed
        display('> handle_after_whois_searcher_server_closed');

        m_result_path:= f_exe_path+ '..\_data\_result\';
        l_base_name:= f_replace_character(m_search_ip, '.', '_')
            + '_'+ IntToStr(m_trial_count)+ '_'+ f_whois_server_key;

        if m_c_result_string_list= Nil
          then handle_no_result
          else test_server_switch;

        display('< handle_after_whois_searcher_server_closed');
      end; // handle_after_whois_searcher_server_closed

    Destructor c_whois_searcher.Destroy;
      begin
        m_c_result_string_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_whois_searcher
    end. // u_c_whois_searcher
