// 002 u_c_whois_server_list_3
// 24 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_whois_server_list_3;
  interface
    uses Classes, u_c_basic_object
        , u_c_base_socket_4
        , u_c_client_socket_4
        ;

    type c_whois_server_list= class; // forward
         t_on_after_whois_event= procedure(p_c_whois_server_list: c_whois_server_list) of Object;

         c_whois_server= // one "whois_server"
             Class(c_basic_object)
               // -- m_name: the domain ("whois.arin.net")
               m_ip: String; // 69.25.34.144
               m_key: String; // us
               m_unavailable: Boolean;
               m_used: Boolean;

               Constructor create_whois_server(p_name: String);
               function f_display_whois_server: String;
               function f_c_self: c_whois_server;
               Destructor Destroy; Override;
             end; // c_whois_server

         c_whois_server_list= // "whois_server" list
             Class(c_basic_object)
               m_c_whois_server_list: tStringList;

               // -- running search index
               m_find_ip_index: Integer;

               m_on_after_create_whois_socket,
                 m_on_after_lookup_whois_socket,
                 m_on_after_changed_whois_server: t_on_after_whois_event;

               m_c_current_whois_server: c_whois_server;

               Constructor create_whois_server_list(p_name: String);

               function f_whois_server_count: Integer;
               function f_c_whois_server(p_whois_server_index: Integer): c_whois_server;
               function f_index_of(p_whois_server_name: String): Integer;
               function f_c_find_by_whois_server_by_name(p_whois_server_name: String): c_whois_server;
               procedure add_whois_server(p_whois_server_name: String; p_c_whois_server: c_whois_server);
               function f_c_add_whois_server(p_whois_server_name: String): c_whois_server;
               function f_c_add_unique_whois_server(p_whois_server_name: String): c_whois_server;
               procedure display_whois_server_list;

                 procedure lookup_whois_server(p_whois_server_name: String);
                 procedure handle_after_socket_error(p_c_base_socket: c_base_socket);
                 procedure handle_after_socket_host_lookup(p_c_client_socket: c_client_socket);
                 procedure search_next_server_ip;
               procedure find_server_ips;

               function f_c_find_whois_server_by_key(p_key: String): c_whois_server;

               procedure set_current_whois_server(p_key: String);
               procedure current_whois_server_timed_out;
               function f_get_next_server: Boolean;
               procedure reset_whois_servers;

               procedure load_whois_servers(p_full_file_name: String);
               procedure load_keys(p_full_file_name: String);

               Destructor Destroy; Override;
             end; // c_whois_server_list

  implementation
    uses SysUtils, u_c_display, u_strings;

    // -- c_whois_server

    Constructor c_whois_server.create_whois_server(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_whois_server

    function c_whois_server.f_display_whois_server: String;
      begin
        Result:= Format('%-30s  %-10s ', [m_name, m_key])+ m_ip;
      end; // f_display_whois_server

    function c_whois_server.f_c_self: c_whois_server;
      begin
        Result:= Self;
      end; // f_c_self

    Destructor c_whois_server.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_whois_server_list

    Constructor c_whois_server_list.create_whois_server_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_whois_server_list:= tStringList.Create;
      end; // create_whois_server_line

    function c_whois_server_list.f_whois_server_count: Integer;
      begin
        Result:= m_c_whois_server_list.Count;
      end; // f_whois_server_count

    function c_whois_server_list.f_c_whois_server(p_whois_server_index: Integer): c_whois_server;
      begin
        Result:= c_whois_server(m_c_whois_server_list.Objects[p_whois_server_index]);
      end; //  f_c_whois_server

    function c_whois_server_list.f_index_of(p_whois_server_name: String): Integer;
      begin
        Result:= m_c_whois_server_list.IndexOf(p_whois_server_name);
      end; // f_index_of

    function c_whois_server_list.f_c_find_by_whois_server_by_name(p_whois_server_name: String): c_whois_server;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_whois_server_name);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_whois_server(m_c_whois_server_list.Objects[l_index_of]);
      end; // f_c_find_by_whois_server_by_name

    procedure c_whois_server_list.add_whois_server(p_whois_server_name: String; p_c_whois_server: c_whois_server);
      begin
        m_c_whois_server_list.AddObject(p_whois_server_name, p_c_whois_server);
      end; // add_whois_server

    function c_whois_server_list.f_c_add_whois_server(p_whois_server_name: String): c_whois_server;
      begin
        Result:= c_whois_server.create_whois_server(p_whois_server_name);
        add_whois_server(p_whois_server_name, Result);
      end; // f_c_add_whois_server

    function c_whois_server_list.f_c_add_unique_whois_server(p_whois_server_name: String): c_whois_server;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_whois_server_name);
        if l_index_of>= 0
          then Result:= f_c_whois_server(l_index_of)
          else Result:= f_c_add_whois_server(p_whois_server_name);
      end; // f_c_add_unique_whois_server

    procedure c_whois_server_list.display_whois_server_list;
      var l_whois_server_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_whois_server_count));

        for l_whois_server_index:= 0 to f_whois_server_count- 1 do
          display(f_c_whois_server(l_whois_server_index).f_display_whois_server);
(*
        with m_c_whois_name_list do
          for l_list_index:= 0 to Count- 1 do
            display(Format('%-10s   %-30s   ',
                [m_c_key_list[l_list_index], Strings[l_list_index]])+ m_c_whois_ip_list[l_list_index]);
*)
      end; // display_whois_server_list

    // -- socket

    procedure c_whois_server_list.lookup_whois_server(p_whois_server_name: String);
        // -- has clicked on a Whois Server name => start the lookup
      begin
        display('> lookup_whois_server');

        with c_client_socket.create_client_socket('whois') do
        begin
          m_trace_socket:= True;

          m_on_after_socket_error:= handle_after_socket_error;
          m_on_after_socket_host_lookup:= handle_after_socket_host_lookup;

          if Assigned(m_on_after_create_whois_socket)
            then m_on_after_create_whois_socket(Self);

          wsa_startup;

          do_lookup_win_socket_server(p_whois_server_name);
        end; // with c_client_socket

        display('< lookup_whois_server');
      end; // lookup_whois_server

    procedure c_whois_server_list.handle_after_socket_error(p_c_base_socket: c_base_socket);
        // -- error during server lookup
      begin
        display('> handle_whois_socket_error');
        with p_c_base_socket do
          display(m_socket_error_text+ ' '+ m_wsa_socket_error_message);
        // -- this whois server is unavailable: remove it from the servers
        m_c_current_whois_server.m_unavailable:= True;

        // -- skip this entry
        p_c_base_socket.Free;
        search_next_server_ip;

        display('< handle_whois_socket_error');
      end; // handle_socket_error

    procedure c_whois_server_list.handle_after_socket_host_lookup(p_c_client_socket: c_client_socket);
        // -- received the IP of the Whois Server
      begin
        display('> handle_after_socket_host_lookup');
        with m_c_current_whois_server do
        begin
          m_ip:= p_c_client_socket.m_remote_ip;
          display('ok '+ m_name+ ' => '+ m_ip);
        end; // with m_c_current_whois_server

        if Assigned(m_on_after_lookup_whois_socket)
          then m_on_after_lookup_whois_socket(Self);

        p_c_client_socket.Free;

        search_next_server_ip;
        display('< handle_after_socket_host_lookup');
      end; // handle_after_whois_lookup

    procedure c_whois_server_list.search_next_server_ip;
      begin
        display('> search_next_server_ip');
        Inc(m_find_ip_index);

        if m_find_ip_index>= f_whois_server_count
          then
          else begin
              m_c_current_whois_server:= f_c_whois_server(m_find_ip_index);
              lookup_whois_server(m_c_current_whois_server.m_name);
            end;
        display('< search_next_server_ip');
      end; // search_next_server_ip

    procedure c_whois_server_list.find_server_ips;
      begin
        display('> find_server_ips');
        m_find_ip_index:= -1;
        // -- create the socket
        search_next_server_ip;
        display('< find_server_ips');
      end; // find_server_ips

    function c_whois_server_list.f_c_find_whois_server_by_key(p_key: String): c_whois_server;
      var l_whois_server_index: Integer;
      begin
        l_whois_server_index:= 0;
        while l_whois_server_index< f_whois_server_count do
          with f_c_whois_server(l_whois_server_index) do
            if m_key= p_key
              then begin
                  Result:= f_c_self;
                  Exit;
                end
              else Inc(l_whois_server_index)  ;

        Result:= Nil;
        display_bug_halt('no_such_key '+ p_key);
      end; // f_c_find_whois_server_by_key

    procedure c_whois_server_list.reset_whois_servers;
      var l_whois_server_index: Integer;
      begin
        for l_whois_server_index:= 0 to f_whois_server_count- 1 do
          f_c_whois_server(l_whois_server_index).m_used:= False;
      end; // reset_whois_servers

    procedure c_whois_server_list.set_current_whois_server(p_key: String);
      begin
        m_c_current_whois_server:= f_c_find_whois_server_by_key(p_key);
        m_c_current_whois_server.m_used:= True;
        display('++ current '+ m_c_current_whois_server.m_key);

        if Assigned(m_on_after_changed_whois_server)
          then m_on_after_changed_whois_server(Self)
          else display('no_after_changed');
      end; // set_current_whois_server

    procedure c_whois_server_list.current_whois_server_timed_out;
      begin
        m_c_current_whois_server.m_unavailable:= True;
      end; // current_whois_server_timed_out

    function c_whois_server_list.f_get_next_server: Boolean;
      var l_whois_server_index: Integer;
      begin
        // -- use another current server
        Result:= False;
        for l_whois_server_index:= 0 to f_whois_server_count- 1 do
          with f_c_whois_server(l_whois_server_index) do
            if m_used or m_unavailable
              then
              else begin
                  set_current_whois_server(m_key);
                  Result:= true;
                  Break;
                end;

        if Result and Assigned(m_on_after_changed_whois_server)
          then m_on_after_changed_whois_server(Self);
      end; // f_get_next_server

    procedure c_whois_server_list.load_whois_servers(p_full_file_name: String);
      var l_list_index: Integer;
          l_the_line: String;
      begin
        with tStringList.Create do
        begin
          LoadFromFile(p_full_file_name);

          for l_list_index:= 0 to Count- 1 do
          begin
            l_the_line:= Trim(Strings[l_list_index]);
            if l_the_line<> ''
              then begin
                  if (l_the_line[1]<> '/')
                    then f_c_add_unique_whois_server(l_the_line);
                end
          end; // while l_list_index

          Free;
        end; // with tStringList
      end; // load_whois_servers

    procedure c_whois_server_list.load_keys(p_full_file_name: String);
      var l_list_index: Integer;
          l_the_line: String;
          l_index: Integer;
          l_server_name, l_server_key: String;
      begin
        display('> load_keys');
        with tStringList.Create do
        begin
          LoadFromFile(p_full_file_name);
          for l_list_index:= 0 to Count- 1 do
          begin
            l_the_line:= Trim(Strings[l_list_index]);
            // display(l_the_line);
            if l_the_line<> ''
              then begin
                  l_index:= 1;
                  l_server_name:= f_string_extract_non_blank(l_the_line, l_index);
                  l_server_key:= f_string_extract_non_blank(l_the_line, l_index);
                  with f_c_find_by_whois_server_by_name(l_server_name) do
                    m_key:= l_server_key;
                end;

          end;
          Free;
        end; // with tStringList
        display('< load_keys');
      end; // load_keys

    Destructor c_whois_server_list.Destroy;
      var l_whois_server_index: Integer;
      begin
        // m_c_server_socket.Free;

        for l_whois_server_index:= 0 to f_whois_server_count- 1 do
          f_c_whois_server(l_whois_server_index).Free;
        m_c_whois_server_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_whois_server_list
    end. // u_c_whois_server_list
